/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ValueHeaders.
 *
 * The Initial Developer of the Original Code is Karsten Düsterloh <mnyromyr@tprac.de>.
 *
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Karsten Düsterloh <mnyromyr@tprac.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gfValueHeadersLoadIdentity = null;
var goValueHeaders = {sPrefBase: "", rsPairs: null};


// Init
function InitValueHeaders()
{ // read the header/value pairs
  if (ReadValueHeaders())
  { // add header/value pairs to addressing widget
    AddValueHeaders(1);
  }
  // We did get here after the initial LoadIdentity -
  // make sure that we don't miss it a second time ;-)
  if (!gfValueHeadersLoadIdentity)
  {
    gfValueHeadersLoadIdentity = LoadIdentity;
    LoadIdentity = ValueHeadersLoadIdentity;
  }

  // add value setter to dropdown
  var oDropDown = awGetPopupElement(1);
  if (oDropDown)
  {
    var sCommand = oDropDown.getAttribute("oncommand");
    sCommand += "; SetValueHeadersValue(this);";
    oDropDown.setAttribute("oncommand", sCommand);
  }

  // Since we have the value coded into the menuitem label, we can't use that
  // as the header name. But the menulist label is always right...
  // HACK!
  // Change the actual function via RegExp...
  var sRecipients2CompFields = uneval(Recipients2CompFields);
  if (sRecipients2CompFields)
  {
    sRecipients2CompFields = sRecipients2CompFields.replace(/(addrOther \+= awGetPopupElement\(i\)\.)selectedItem\./, "$1");
    eval("Recipients2CompFields=" + sRecipients2CompFields, window);
  }
}


// Add the header/value pairs to the addressing widget      
function AddValueHeaders(aiRow)
{ 
  if (goValueHeaders.rsPairs && goValueHeaders.rsPairs.length)
  {
    var oDropDown = awGetPopupElement(aiRow);
    if (oDropDown)
      for (var i = 0; i < goValueHeaders.rsPairs.length; ++i)
        oDropDown.appendItem(goValueHeaders.rsPairs[i], "addr_other");
  }
}


// if one of our header/value pairs is selected, strip the value from the dropdown
// and move it into the textbox
function SetValueHeadersValue(aoNode)
{
  var sValue = aoNode.label.replace(/^.*?:\s*/, "");
  if (sValue)
  { // dat's one o' us! ;-)
    var iRow = aoNode.id.slice(aoNode.id.lastIndexOf('#') + 1);
    var oTextBox = awGetInputElement(iRow);
    if (oTextBox)
    {
      oTextBox.value = sValue;
      aoNode.setAttribute("label", aoNode.label.replace(/:.*$/, ":"));
    }
  }
}


// remove all whitespace and capitalize word beginnings; delimiters are "-,:"
function PrettifyHeader(asHeader)
{
  asHeader = asHeader.toLowerCase().replace(/\s/g, "");
  asHeader = asHeader.replace(/(^[a-z])|([^a-z][a-z])/g, function(s){return s.toUpperCase()});
  return asHeader;
}


// get the base preference name and the respective list of header/value pairs
function ReadValueHeaders(asKey)
{
  var bDirty = false;
  if (goValueHeaders)
  {
    if (!asKey)
      asKey = gCurrentIdentity.key;
    
    var sPrefBase = "";
    var sPrefList = "";
  
    try 
    {
      var oPrefs = Components.classes["@mozilla.org/preferences-service;1"]
                             .getService(Components.interfaces.nsIPrefBranch);
      if (oPrefs)
      { // get the header names and values for the current identity
        // if there are none defined, try the default identity
        try 
        {
          sPrefBase = "mail.identity." + asKey + ".compose.header";
          sPrefList = oPrefs.getCharPref(sPrefBase + "s");
        }
        catch(oIgnore){}
        if (!sPrefList)
        {
          try 
          {
            sPrefBase = "mail.identity.default.compose.header";
            sPrefList = oPrefs.getCharPref(sPrefBase + "s");
          }
          catch(oIgnore){}
        }
      }
    }
    catch(oError)
    {
      dump("InitValueHeaders: no Pref service\n")
    }

    bDirty = sPrefBase != goValueHeaders.sPrefBase;
    if (bDirty)
    { // we need to rebuild our array of header/value pairs
      goValueHeaders.sPrefBase = sPrefBase;
      goValueHeaders.rsPairs   = [];
      if (sPrefList)
      { // loop over all the promised headers
        var rsHeaders = sPrefList.split(",");
        var sPref, sHeader, rsHeaderValue, sName, sValue;
        for (var i = 0; i < rsHeaders.length; ++i)
        {
          sPref = rsHeaders[i];
          if (sPref)
          { // get the header/value pair             
            try
            {
              sHeader = oPrefs.getCharPref(sPrefBase + "." + sPref);
            }
            catch(oIgnore){}
            // add it to the dropdown
            if (sHeader)
            {
              rsHeaderValue = sHeader.split(":");
              sName  = PrettifyHeader(rsHeaderValue.shift());
              sValue = rsHeaderValue.join(":")
              if (sName && sValue)
                goValueHeaders.rsPairs.push(sName + ": " + sValue);
            }
          } // have header pref
        } // loop over prefs
      } // new list
    } // have new headers
  } // have goValueHeaders
  return bDirty;
}


// Remove our old header/value pairs from the address widgets
// and replace them by our new ones
function UpdateAddressWidget(asIDKey)
{
  if (ReadValueHeaders(asIDKey))
  { // yes, change all dropdowns
    var oDropDown, oItems, oItem;
    for (var i = 1; oDropDown = awGetPopupElement(i); ++i)
    { // if this row is one of ours -> deselect it and kill text
      if (oDropDown.selectedItem.label.match(/:.+$/))
      {
        oDropDown.selectedIndex = 0;
        awGetInputElement(i).value = "";
      }
      // not one of us (yet ;-) ) 
      // clean up the widget
      if (oDropDown.menupopup)
      {
        oItems = oDropDown.menupopup.childNodes;
        for (var j = oItems.length - 1; j >=0; --j)
        {
          oItem = oItems.item(j);
          // if this item is one of ours -> delete it
          if (oItem.label.match(/:.+$/))
            oDropDown.removeItemAt(j);
        } // for all headers in dropdown
      }
      // add new ones
      AddValueHeaders(i);
    } // for all rows
  } //ID changed
}



function ValueHeadersLoadIdentity(abStartup)
{ // we only need to look closer if the ID did change
  var oIdentityNode = document.getElementById("msgIdentity");
  if (oIdentityNode) 
  {
    var oPrevIdentity = gCurrentIdentity;
    var sIDKey = oIdentityNode.value;
    var oCurrentIdentity = gAccountManager.getIdentity(sIDKey);
    if (oPrevIdentity && oPrevIdentity.key != sIDKey)
    { // so the ID did change, but did our gsBasePref?
      UpdateAddressWidget(sIDKey);
    }
  } // have node
  if (gfValueHeadersLoadIdentity)
    gfValueHeadersLoadIdentity(abStartup);
}


addEventListener('compose-window-reopen', UpdateAddressWidget, true);


